@@ -1,4 +1,4 @@
-from concurrent.futures import ThreadPoolExecutor
+from concurrent.futures import ThreadPoolExecutor, as_completed
 from functools import partial
 from multiprocessing import cpu_count
 from typing import Any, Callable, List, Optional
@@ -12,14 +12,24 @@ class ThreadedFunction(ParalellyFunction):
         return self._func(*args, **kwargs)
 
     def map(self, *args, **kwargs) -> List[Any]:
+        return list(self.imap(*args, **kwargs))
+
+    def imap(self, *args, **kwargs) -> List[Any]:
         args, kwargs = prepare_arguments(args, kwargs)
         pool_size = min(self._max_workers, len(args))
 
         with ThreadPoolExecutor(pool_size) as pool:
             futures = [pool.submit(self._execute_once, *arg, **kwarg) for arg, kwarg in zip(args, kwargs)]
+            for future in futures:
+                yield future.result() 
 
-        return [future.result() for future in futures]
+    def imap_gready(self, *args, **kwargs) -> List[Any]:
+        args, kwargs = prepare_arguments(args, kwargs)
+        pool_size = min(self._max_workers, len(args))
 
+        with ThreadPoolExecutor(pool_size) as pool:
+            for future in as_completed([pool.submit(self._execute_once, *arg, **kwarg) for arg, kwarg in zip(args, kwargs)]):
+                yield future.result()
 
 def threaded(func: Callable = None, max_workers: Optional[int] = None) -> ThreadedFunction:
     """
